library ieee;
use ieee.std_logic_1164.all;

entity part4 is
	generic (
		-- codifica simboli ('0' -> punto, '1' -> linea), LSB zero fill a 4 bit
		CH0_MORSE : std_logic_vector(3 downto 0) := "0100"; -- carattere A (.-)
		CH1_MORSE : std_logic_vector(3 downto 0) := "1000"; -- carattere B (-...)
		CH2_MORSE : std_logic_vector(3 downto 0) := "1010"; -- carattere C (-.-.)
		CH3_MORSE : std_logic_vector(3 downto 0) := "1000"; -- carattere D (-..)
		CH4_MORSE : std_logic_vector(3 downto 0) := "0000"; -- carattere E (.)
		CH5_MORSE : std_logic_vector(3 downto 0) := "0010"; -- carattere F (..-.)
		CH6_MORSE : std_logic_vector(3 downto 0) := "1100"; -- carattere G (--.)
		CH7_MORSE : std_logic_vector(3 downto 0) := "0000"; -- carattere H (....)
		-- lunghezza simboli
		CH0_LENGTH : integer := 2; -- carattere A
		CH1_LENGTH : integer := 4; -- carattere B
		CH2_LENGTH : integer := 4; -- carattere C
		CH3_LENGTH : integer := 3; -- carattere D
		CH4_LENGTH : integer := 1; -- carattere E
		CH5_LENGTH : integer := 4; -- carattere F
		CH6_LENGTH : integer := 3; -- carattere G
		CH7_LENGTH : integer := 4; -- carattere H
		NUMCYCLES: integer:=25000000); -- tempo di ciclo 0.5 s (periodi a 50MHz)
	port(
		SW: in std_logic_vector(9 downto 0);
		KEY: in std_logic_vector(2 downto 0);
		CLOCK_50 : in std_logic;
		LEDG: out std_logic_vector(9 downto 0));
end part4;

architecture arch_part4 of part4 is

component Tcounter
	generic(ENDCOUNT: integer:=255);
	port(
		clk, ares: in std_logic;
		countwrap: out std_logic);
end component;

component shreg
	generic(LEN: integer:=8);
	port(
		clk, ares: in std_logic;
		load, enable: in std_logic;
		PARIN: in std_logic_vector(LEN-1 downto 0);
		serout: out std_logic);
end component;

component Scounter
	port(
		clk, ares: in std_logic;
		load, enable: in std_logic;
		COUNTIN: in integer;
		COUNTOUT: buffer integer);
end component;

component FSM
	port(
		clk, ares: in std_logic;
		Tstep, start, symbol: in std_logic;
		NSYM: in integer;
		load_sym, next_sym: out std_logic;
		morse_out: out std_logic);
end component;

signal reset,clk,Tstep: std_logic;
signal CHAR_SEL: std_logic_vector(2 downto 0);
signal MORSE_CODE: std_logic_vector(3 downto 0);
signal MORSE_LENGTH: integer;
signal start, symbol, morse_out: std_logic;
signal NSYMBOLS: integer;
signal load_reg, enable_reg: std_logic;
signal STATE_OUT: std_logic_vector(5 downto 0);

begin

	selection: process (CHAR_SEL)
	begin
		case CHAR_SEL is
			when "000" =>
				MORSE_CODE <= CH0_MORSE;
				MORSE_LENGTH <= CH0_LENGTH;
			when "001" =>
				MORSE_CODE <= CH1_MORSE;
				MORSE_LENGTH <= CH1_LENGTH;
			when "010" =>
				MORSE_CODE <= CH2_MORSE;
				MORSE_LENGTH <= CH2_LENGTH;
			when "011" =>
				MORSE_CODE <= CH3_MORSE;
				MORSE_LENGTH <= CH3_LENGTH;
			when "100" =>
				MORSE_CODE <= CH4_MORSE;
				MORSE_LENGTH <= CH4_LENGTH;
			when "101" =>
				MORSE_CODE <= CH5_MORSE;
				MORSE_LENGTH <= CH5_LENGTH;
			when "110" =>
				MORSE_CODE <= CH6_MORSE;
				MORSE_LENGTH <= CH6_LENGTH;
			when "111" =>
				MORSE_CODE <= CH7_MORSE;
				MORSE_LENGTH <= CH7_LENGTH;
		end case;
	end process;

	Tcounter0: Tcounter
		generic map(NUMCYCLES-1)
		port map(clk,reset,Tstep);

	shreg0: shreg
		generic map(4)
		port map(clk,reset,load_reg,enable_reg,MORSE_CODE,symbol);

	Scounter0: Scounter
		port map(clk,reset,load_reg,enable_reg,MORSE_LENGTH,NSYMBOLS);

	FSM0: FSM
		port map(clk,reset,Tstep,not start,symbol,NSYMBOLS,load_reg,enable_reg,morse_out);
	
	reset <= KEY(0);
	clk <= CLOCK_50;
	CHAR_SEL <= SW (2 downto 0);
	start <= KEY(1);
	LEDG(0) <= morse_out;
	
end arch_part4;

----------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity Tcounter is
	generic(ENDCOUNT: integer:=255);
	port(
		clk, ares: in std_logic;
		countwrap: out std_logic);
end Tcounter;

architecture arch_Tcounter of Tcounter is
signal count: integer;
begin
	
	process (clk, ares)
	begin
		if ares = '0' then
			count <= 0;
			countwrap <= '0';
		elsif clk'event and clk = '1' then
			if count = ENDCOUNT then
				count <= 0;
				countwrap <= '1';
			else
				count <= count + 1;
				countwrap <= '0';
			end if;
		end if;
	end process;
	
end arch_Tcounter;

----------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity shreg is
	generic(LEN: integer:=8);
	port(
		clk, ares: in std_logic;
		load, enable: in std_logic;
		PARIN: in std_logic_vector(LEN-1 downto 0);
		serout: out std_logic);
end shreg;

architecture arch_shreg of shreg is
signal REG: std_logic_vector(LEN-1 downto 0);
begin
	
	process (clk, ares)
	begin
		if ares = '0' then
			REG <= (others => '0');
		elsif clk'event and clk = '1' then
			if load = '1' then
				REG <= PARIN;
			elsif enable = '1' then
				REG <= REG(LEN-2 downto 0) & '0';
			end if;
		end if;
	end process;
	
	serout <= REG(LEN-1);
	
end arch_shreg;

----------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity Scounter is
	port(
		clk, ares: in std_logic;
		load, enable: in std_logic;
		COUNTIN: in integer;
		COUNTOUT: buffer integer);
end Scounter;

architecture arch_Scounter of Scounter is
begin
	
	process (clk, ares)
	begin
		if ares = '0' then
			COUNTOUT <= 0;
		elsif clk'event and clk = '1' then
			if load = '1' then
				COUNTOUT <= COUNTIN;
			elsif enable = '1' then
				if COUNTOUT = 0 then
					COUNTOUT <= 0;
				else
					COUNTOUT <= COUNTOUT-1;
				end if;
			end if;
		end if;
	end process;
	
end arch_Scounter;

----------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity FSM is
	port(
		clk, ares: in std_logic;
		Tstep, start, symbol: in std_logic;
		NSYM: in integer;
		load_sym, next_sym: out std_logic;
		morse_out: out std_logic);
end FSM;

architecture arch_FSM of FSM is
type state is (s_WAIT_SEND, s_WAIT_BLANK,	s_SEND_DOT, s_SEND_DASH1,
					s_SEND_DASH2, s_SEND_DASH3, s_RELEASE_SEND);
signal present_state : state;

begin
	
	process (clk, ares)
	begin
		if ares = '0' then
			present_state <= s_WAIT_SEND;
		elsif clk'event and clk = '1' then
			case present_state is
			
				when s_WAIT_SEND =>
					if start = '1' then
						present_state <= s_WAIT_BLANK;
					else
						present_state <= s_WAIT_SEND;
					end if;
					
				when s_WAIT_BLANK =>
					if Tstep = '1' then
						if symbol = '0' then
							present_state <= s_SEND_DOT;
						else
							present_state <= s_SEND_DASH1;
						end if;
					else
						present_state <= s_WAIT_BLANK;
					end if;
				
				when s_SEND_DOT =>
					if Tstep = '1' then 
						if NSYM = 1 then
							present_state <= s_RELEASE_SEND;
						else
							present_state <= s_WAIT_BLANK;
						end if;
					else
						present_state <= s_SEND_DOT;
					end if;
				
				when s_SEND_DASH1 =>
					if Tstep = '1' then
						present_state <= s_SEND_DASH2;
					else
						present_state <= s_SEND_DASH1;
					end if;
				
				when s_SEND_DASH2 =>
					if Tstep = '1' then
						present_state <= s_SEND_DASH3;
					else
						present_state <= s_SEND_DASH2;
					end if;
				
				when s_SEND_DASH3 =>
					if Tstep = '1' then
						if NSYM = 1 then
							present_state <= s_RELEASE_SEND;
						else
							present_state <= s_WAIT_BLANK;
						end if;
					else
						present_state <= s_SEND_DASH3;
					end if;
				
				when s_RELEASE_SEND =>				
					if start = '1' then
						present_state <= s_RELEASE_SEND;
					else
						present_state <= s_WAIT_SEND;
					end if;
				
				when others =>
					present_state <= s_WAIT_SEND;
					
			end case;
		end if;
	end process;

	load_sym <= '1' when present_state = s_WAIT_SEND and start = '1' else '0';
	
	next_sym <= '1' when (present_state = s_SEND_DOT or present_state = s_SEND_DASH3)
		and Tstep = '1' else '0';
	
	morse_out <= '1' when present_state = s_SEND_DOT or present_state = s_SEND_DASH1 
		or present_state = s_SEND_DASH2 or present_state = s_SEND_DASH3 else '0';

end arch_FSM;